import type { LanguageInput, ThemedToken, ThemeInput, ThemeRegistrationAny, TokensResult } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'

const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface SyntaxHighlighterProps {
  text: string
  language?: LanguageInput
  languageId?: string
  theme: ThemeRegistrationAny
  fontSize?: number
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const engine = createJavaScriptRegexEngine({
  forgiving: true,
  simulation: true,
  cache: new Map(),
})

async function createHighlighter({ langs, themes }: { langs: LanguageInput[], themes: ThemeInput[] }) {
  if (!langs?.length) {
    throw new Error('Please provide languages when creating a highlighter.')
  }

  try {
    return await createHighlighterCore({
      langs,
      themes,
      engine,
    })
  }
  catch (error) {
    console.error('Failed to create highlighter:', error)
    throw error
  }
}

function parseColor(color?: string): string | undefined {
  if (!color)
    return undefined

  // Handle hex with alpha (#RRGGBBAA)
  if (color.length === 9 && color.startsWith('#')) {
    const alpha = Number.parseInt(color.slice(7, 9), 16) / 255
    const r = Number.parseInt(color.slice(1, 3), 16)
    const g = Number.parseInt(color.slice(3, 5), 16)
    const b = Number.parseInt(color.slice(5, 7), 16)
    return `rgba(${r}, ${g}, ${b}, ${alpha})`
  }

  return color
}

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<Error | null>(null)

  // Generate styles once when theme changes
  const styles = useMemo(() => {
    const backgroundColor = parseColor(theme.colors?.['editor.background']) || '#1e1e1e'
    const foregroundColor = parseColor(theme.colors?.['editor.foreground']) || '#d4d4d4'

    return StyleSheet.create({
      scrollView: {
        flex: 1,
        minHeight: 20,
      },
      codeContainer: {
        padding: 16,
        minWidth: '100%',
        backgroundColor,
      },
      lineContainer: {
        flexDirection: 'row',
        flexWrap: 'wrap',
      },
      token: {
        fontFamily: FONT_FAMILY,
        color: foregroundColor,
      },
    })
  }, [theme])

  // Cache token styles
  const styleCache = useMemo(() => new Map<string, TextStyle>(), [])

  const getTokenStyle = (token: ThemedToken): TextStyle => {
    const key = `${token.color || ''}-${token.fontStyle || ''}`

    if (!styleCache.has(key)) {
      styleCache.set(key, {
        color: parseColor(token.color) || styles.token.color,
        fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
        fontWeight: token.fontStyle === 1 ? 'bold' : 'normal',
      })
    }

    return styleCache.get(key)!
  }

  // Highlight code when text/language changes
  useEffect(() => {
    let mounted = true

    async function highlight() {
      if (!text) {
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighter({
          langs: language ? [language] : [],
          themes: [theme],
        })

        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme,
        }) as TokensResult

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(null)
        }
      }
      catch (err) {
        console.error('Syntax highlighting failed:', err)
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'))
          // Fallback to plain text display on error
          setTokens([[{ content: text, color: styles.token.color, offset: 0 }]])
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  if (error) {
    // Fallback to plain text on error
    return (
      <ScrollView horizontal {...scrollViewProps} style={[styles.scrollView, scrollViewProps?.style]} contentContainerStyle={[styles.codeContainer, containerStyle]}>
        <Text style={[styles.token, { fontSize }, textStyle]}>{text}</Text>
      </ScrollView>
    )
  }

  return (
    <ScrollView horizontal {...scrollViewProps} style={[styles.scrollView, scrollViewProps?.style]} contentContainerStyle={[styles.codeContainer, containerStyle]}>
      <View>
        {tokens.map((line, lineIndex) => (
          <View key={lineIndex} style={styles.lineContainer}>
            {line.map((token, tokenIndex) => (
              <Text key={`${lineIndex}-${tokenIndex}`} style={[styles.token, getTokenStyle(token), { fontSize }, textStyle]}>
                {token.content}
              </Text>
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
